package ru.flexfactory.controller
{
	import flash.events.*;
	import ru.flexfactory.controller.operations.MathOperation;
	import ru.flexfactory.model.*;

	public class AppController extends EventDispatcher
	{
		public static const OPERATOR_CHANGE:String = "Operator change";
		
		private var appData:AppData;                        //объект для оперирования значениями
		private var memory:Memory;                          //объект для оперирования ячейкой памяти
		
		private var lastOperationIsResult:Boolean;          //флаг, указывающий, была ли последняя операция подсчётом итогового значения
		private var lastButtonIsNumeric:Boolean;            //флаг, указывающий, была ли последняя нажатая кнопка - цифровой кнопкой 
		
		public function AppController()
		{
			appData = new AppData();
			memory = new Memory();
			
			clear();
		}

		//нажатие цифровой кнопки 
		//после изменения отображаемого значения отправляем событие о том, что значение изменилось
		//отправлять событие мы будем при любом изменении данных либо памяти
		public function pressNumericButton(digit:String):void
		{
			if (!lastButtonIsNumeric) appData.clearDisplayValue();
			appData.addDigit(digit);
			sendEvent();
			
			lastButtonIsNumeric = true;
		}
		
		//нажатие кнопки с точкой
		//т.к. точка является частью числа, то флаг lastButtonIsNumeric устанавливаем в true
		public function pressDotButton():void
		{
			appData.addComma();
			sendEvent();
			
			lastButtonIsNumeric = true;
		}
		
		//нажатие кнопки знака "-"
		public function pressSignButton():void
		{
			appData.changeSign();
			sendEvent();
			
			lastButtonIsNumeric = true;
		}
		
		//нажатие кнопки математической операции
		//тут в зависимости от состояния флагов производим вычисление
		public function pressOperationButton(operation:MathOperation):void
		{
			calcForOperation();
			appData.setOperation(operation);
			
			lastButtonIsNumeric = false;
			lastOperationIsResult = false;
		}
		
		//действие, если нажата кнопка математической операции
		//я вынес этот код в отдельную функцию, чтобы соблюдать правило "Одна функция - одно действие"
		private function calcForOperation():void
		{
			if (lastOperationIsResult) 
			{
				appData.displayToX();
			}
			else
			{
				if (lastButtonIsNumeric)
				{
					appData.displayToY();
					appData.calculate();
					sendEvent();
				}
			}
		}
		
		//нажатие кнопки [=]
		public function pressResultButton():void
		{
			if (!lastOperationIsResult) appData.displayToY();
			appData.calculate();
			sendEvent();
			
			lastOperationIsResult = true;
			lastButtonIsNumeric = false;
		}
		
		//вычисление квадратного корня
		//возможна ошибка, если была попытка вычислить корень из отрицательного числа
		public function pressSquareButton():void
		{
			try 
			{
				appData.square();
				pressResultButton();
			}
			catch (e:Error)
			{
				sendEvent();
			}
		}
		
		//нажатие на кнопку [%]
		public function pressPercentButton():void
		{
			appData.percent();
			pressResultButton();
		}
		
		//нажатие на кнопки MEMORY
		//при добавлении или вычитании значения к значению в памяти, производим вычисление, как будто была нажата кнопка [=]
		public function pressMemoryClear():void
		{
			memory.clear();
			sendEvent();
		}
		
		public function pressMemoryRead():void
		{
			appData.displayed = memory.read().toString();
			sendEvent();
			lastButtonIsNumeric = true;
		}
		
		public function pressMemoryAdd():void
		{
			pressResultButton();
			memory.add(Number(appData.displayed));
			sendEvent();
		}
		
		public function pressMemorySub():void
		{
			pressResultButton();
			memory.sub(Number(appData.displayed));
			sendEvent();
		}
		
		//отправляем событие
		private function sendEvent():void
		{
			var event:Event = new Event(OPERATOR_CHANGE);
			dispatchEvent(event);
		}
		
		//метод для получения значения вывода на экран
		public function get displayValue():String
		{
			return appData.displayed;
		}
		
		//метод для вывода буквы "М", если значение в памяти отличное от нуля
		public function get memoryState():String
		{
			return (memory.read() > 0) ? "M" : "";
		}
		
		//сброс значений
		public function clear():void
		{
			appData.clear();
			lastOperationIsResult = true;
			lastButtonIsNumeric = false;
			sendEvent();
		}
	}
}