package ru.flexfactory.model
{
	import flash.errors.IllegalOperationError;
	import ru.flexfactory.controller.operations.MathOperation;
	
	public class AppData
	{
		private const MAX_DIGITS:int = 10;
		private const ZERO_VALUE:String = "0"; 
		private const COMMA:String = "."; 
		private const NEGATION_SIGN:String = "-";
		
		private var x:Number;
		private var y:Number;
		private var operation:MathOperation;
		private var displayValue:String;
		private var valueIsDecimal:Boolean;
		
		public function AppData()
		{
			clear();
		}
		
		//установка значений по-умолчанию для всех переменных
		public function clear():void
		{
			x = 0;
			y = 0;
			operation = null;
			clearDisplayValue();
		}
		
		//добавление числа в строковом виде справа к уже составленному числу
		public function addDigit(digit:String):void
		{
			if (displayValue == ZERO_VALUE) displayValue = "";
			if (lenghtIsValid()) displayValue += digit;
		}
		
		//проверка на достижение максимального разряда.
		//в подсчёте не учитываются знаки "." и "-"
		private function lenghtIsValid():Boolean
		{
			var subDigit:int = 0;
			if (displayValue.indexOf(COMMA) >= 0) subDigit++;
			if (displayValue.substr(0, 1) == NEGATION_SIGN) subDigit++;
			
			return ((displayValue.length - subDigit) < MAX_DIGITS) ? true : false;
		}
		
		//добавление разделителя
		//т.к. это можно сделать только один раз, устанавливаем флаг
		public function addComma():void
		{
			if (!valueIsDecimal)
			{
				displayValue += COMMA;
				valueIsDecimal = true;
			}
		}
		
		//изменение знака числа        
		public function changeSign():void
		{
			var temp:Number = Number(displayValue);
			temp *= -1;
			displayValue = temp.toString();
		}
		
		//очистка значение, которое выводится на дисплей        
		public function clearDisplayValue():void
		{
			displayValue = ZERO_VALUE;
			valueIsDecimal = false;
		}
		
		//записываем число на дисплее в переменную х        
		public function displayToX():void
		{
			x = Number(displayValue);
		}
		
		//записываем число на дисплее в переменную у
		public function displayToY():void
		{
			y = Number(displayValue);
		}
		
		//запоминаем операцию, которую будем выполнять
		//заметим, что тут мы используем только интерфейс, ничего не зная о реализованных классах        
		public function setOperation(operation:MathOperation):void
		{
			this.operation = operation;
		}
		
		//вычисление значения
		//используем конструкцию try...catch, чтобы перехватить ошибку        
		public function calculate():void
		{
			if (operation == null) return;
			
			try
			{
				x = operation.calculate(x, y);
				displayValue = x.toString();
			}
			catch (e:IllegalOperationError)
			{
				displayValue = e.message;
			}
		}
		
		//вычисление корня значения
		//обрабатываем исключение типа "корень из отрицательного числа"
		public function square():void
		{
			if (Number(displayValue) < 0) {
				displayValue = "Не выполнимо";
				throw new Error(displayValue);
			}
			
			var temp:Number = Number(displayValue);
			temp = Math.sqrt(temp);
			displayValue = temp.toString();
		}
		
		//вычисление процента        
		public function percent():void
		{
			var temp:Number = Number(displayValue);
			temp = x / 100 * temp;
			displayValue = temp.toString();
			displayToY();
		}
		
		//setter и getter для локальной переменной displayValue
		public function set displayed(value:String):void
		{
			displayValue = value;
		}
		
		public function get displayed():String
		{
			return displayValue;
		}
	}
}